@@ -13,6 +13,7 @@ import numpy as np
 import torch
 import os
 import warnings
+import shutil
 
 from hummingbird.ml.exceptions import ConstantError
 
@@ -121,7 +122,6 @@ def tvm_installed():
     """
     try:
         import tvm
-        from tvm import relay
     except ImportError:
         return False
     return True
@@ -185,12 +185,10 @@ def load(location):
     """
     Utility function used to load arbitrary Hummingbird models.
     """
-    # Add load capabilities
-    from ._container import PyTorchSklearnContainer
-    from ._container import TVMSklearnContainer
-    from ._container import ONNXSklearnContainer
-
-    import shutil
+    # Add load capabilities.
+    from hummingbird.ml.containers import PyTorchSklearnContainer
+    from hummingbird.ml.containers import TVMSklearnContainer
+    from hummingbird.ml.containers import ONNXSklearnContainer
     from hummingbird.ml.operator_converters import constants
 
     model = None
@@ -201,7 +199,7 @@ def load(location):
     if not location.endswith("zip"):
         zip_location = location + ".zip"
     else:
-        location = zip_location[-4]
+        location = zip_location[:-4]
     shutil.unpack_archive(zip_location, location, format="zip")
 
     assert os.path.exists(location), "Model location {} does not exist.".format(location)
