@@ -26,12 +26,22 @@ def convert_onnx_array_feature_extractor(operator, device, extra_config):
     Returns:
         A PyTorch model
     """
+    assert operator is not None, "Cannot convert None operator"
 
-    # TODO, this will be tested as part of the ai.onnx.ml.OneHotEncoder tests
     column_indices = []
     initializers = extra_config[constants.ONNX_INITIALIZERS]
-    column_indices = initializers[operator.raw_operator.origin.input[1]].int64_data
-    return ArrayFeatureExtractor(column_indices, device)
+    operator_inputs = operator.raw_operator.origin.input
+    column_indices = None
+    for input_ in operator_inputs:
+        if input_ in initializers:
+            assert column_indices is None, "More than one ArrayFeatureExtractor input matches with stored initializers."
+            column_indices = list(initializers[input_].int64_data)
+            if len(column_indices) == 0:
+                # If we are here it means that the column indices were not int64.
+                column_indices = list(initializers[input_].int32_data)
+            assert len(column_indices) > 0, "Cannot convert ArrayFeatureExtractor with empty column indices."
+
+    return ArrayFeatureExtractor(operator, column_indices, device)
 
 
 register_converter("ONNXMLArrayFeatureExtractor", convert_onnx_array_feature_extractor)
