@@ -87,9 +87,7 @@ def convert_sklearn_xgb_classifier(operator, device, extra_config):
     Returns:
         A PyTorch model
     """
-    assert operator is not None
-
-    # Get tree information out of the model.
+    assert operator is not None, "Cannot convert None operator"
     if "n_features" in extra_config:
         n_features = extra_config["n_features"]
     else:
@@ -100,7 +98,9 @@ def convert_sklearn_xgb_classifier(operator, device, extra_config):
     tree_infos = operator.raw_operator.get_booster().get_dump()
     n_classes = operator.raw_operator.n_classes_
 
-    return convert_gbdt_classifier_common(tree_infos, _get_tree_parameters, n_features, n_classes, extra_config=extra_config)
+    return convert_gbdt_classifier_common(
+        operator, tree_infos, _get_tree_parameters, n_features, n_classes, extra_config=extra_config
+    )
 
 
 def convert_sklearn_xgb_regressor(operator, device, extra_config):
@@ -115,7 +115,7 @@ def convert_sklearn_xgb_regressor(operator, device, extra_config):
     Returns:
         A PyTorch model
     """
-    assert operator is not None
+    assert operator is not None, "Cannot convert None operator"
     if "n_features" in extra_config:
         n_features = extra_config["n_features"]
     else:
@@ -124,7 +124,6 @@ def convert_sklearn_xgb_regressor(operator, device, extra_config):
              Please pass "n_features:N" as extra configuration to the converter or fill a bug report.'
         )
 
-    # Get tree information out of the model.
     tree_infos = operator.raw_operator.get_booster().get_dump()
     base_prediction = operator.raw_operator.base_score
     if base_prediction is None:
@@ -134,7 +133,7 @@ def convert_sklearn_xgb_regressor(operator, device, extra_config):
 
     extra_config[constants.BASE_PREDICTION] = base_prediction
 
-    return convert_gbdt_common(tree_infos, _get_tree_parameters, n_features, extra_config=extra_config)
+    return convert_gbdt_common(operator, tree_infos, _get_tree_parameters, n_features, extra_config=extra_config)
 
 
 # Register the converters.
